//
// Parago Media GmbH & Co. KG, Jrgen Burle (jbaurle@parago.de)
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
// TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
// CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
//

#region Using Directives

using System;
using System.Runtime.InteropServices;
using WinForms = System.Windows.Forms;
using Extensibility;
using Microsoft.Office.Core;
using Microsoft.Office.Interop.Excel;
using Microsoft.Win32;
using ParagoStockQuote.UI.Properties;

#endregion

namespace ParagoStockQuote
{
   [Guid(UserInterface.TypeGuid)]
   [ProgId(UserInterface.TypeProgId)]
   [ClassInterface(ClassInterfaceType.AutoDual)]
   public sealed class UserInterface : IDTExtensibility2, IRibbonExtensibility
   {
      #region Constants & Fields

      public const string TypeGuid = "02EF4BAA-7CDF-4030-8109-A9B73EF73161";
      public const string TypeProgId = "ParagoStockQuote.UI";
      public const string ClsIdKeyName = @"CLSID\{" + UserInterface.TypeGuid + @"}\";
      public const string ExcelAddInKeyName = @"Software\Microsoft\Office\Excel\Addins\" + UserInterface.TypeProgId;

      Application _excel;

      #endregion

      #region Initialization

      public UserInterface()
      {
      }

      #endregion

      #region COM Registration

      [ComRegisterFunctionAttribute]
      public static void RegisterFunction(Type t)
      {
         RegistryKey key;

         key = Registry.ClassesRoot.CreateSubKey(UserInterface.ClsIdKeyName + "Programmable");
         key.Close();

         key = Registry.ClassesRoot.CreateSubKey(UserInterface.ClsIdKeyName + "InprocServer32");
         key.SetValue(string.Empty, Environment.SystemDirectory + @"\mscoree.dll");
         key.Close();

         key = Registry.CurrentUser.CreateSubKey(UserInterface.ExcelAddInKeyName);
         key.SetValue("Description", "Parago.de Stock Quotes UI Add-In for Excel 2007", RegistryValueKind.String);
         key.SetValue("FriendlyName", "Parago.de Stock Quotes UI Add-In", RegistryValueKind.String);
         key.SetValue("LoadBehavior", 3, RegistryValueKind.DWord);
         key.SetValue("CommandLineSafe", 0, RegistryValueKind.DWord);
         key.Close();
      }

      [ComUnregisterFunctionAttribute]
      public static void UnregisterFunction(Type t)
      {
         Registry.ClassesRoot.DeleteSubKey(UserInterface.ClsIdKeyName + "Programmable");
         Registry.ClassesRoot.DeleteSubKeyTree(UserInterface.ClsIdKeyName + "InprocServer32");

         Registry.CurrentUser.DeleteSubKey(UserInterface.ExcelAddInKeyName);
      }

      #endregion

      #region IDTExtensibility2 Member

      public void OnAddInsUpdate(ref Array custom)
      {
      }

      public void OnBeginShutdown(ref Array custom)
      {
      }

      public void OnConnection(object host, ext_ConnectMode connectMode, object addInInst, ref Array custom)
      {
         _excel = (Application)host;
      }

      public void OnDisconnection(ext_DisconnectMode removeMode, ref Array custom)
      {
      }

      public void OnStartupComplete(ref Array custom)
      {
      }

      #endregion

      #region IRibbonExtensibility Member

      public string GetCustomUI(string ribbonID)
      {
         return Resources.Ribbon;
      }

      public void OnUpdateButtonClick(IRibbonControl control)
      {
         _excel.CalculateFull();
      }

      public void OnAboutButtonClick(IRibbonControl control)
      {
         WinForms.MessageBox.Show("To retrieve stock quotes see the following formula samples:\n\n\t=PSQ(A1;\"PRICE\") or just =PSQ(A1)\n\t=PSQ(A1;\"DATE\")\n\t=PSQ(A1;\"TIME\"\n\t=PSQ(A1;\"NAME\")\n\nThe Stock Quotes Add-In is developed by Jrgen Burle, http://www.parago.de/jbaurle.\n\nTo contact me, please email: jbaurle@parago.de.", "Parago.de Stock Quotes Add-In", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Information);
      }
      
      #endregion
   }
}
